'use strict';

const chai          = require('chai');
const sinonChai     = require('sinon-chai');
const _             = require('lodash');
const DeployBuilder = require('./../deploy.builder');
const Deploy        = require('./../deploy');

const expect = chai.expect;
chai.use(sinonChai);



describe('Deploy builder unit test', () => {

    describe('construct', () => {
        const tests = [
            {
                title: 'simple',
                descriptor: {
                    metadata: {
                        name: 'fake',
                    },
                    spec: {
                        template: {
                            spec: {
                                containers: [
                                    {
                                        image: 'mongo'
                                    }
                                ]
                            }
                        }
                    }
                },
                expectedObject: {
                    "apiVersion": "extensions/v1beta1",
                    "kind": "Deployment",
                    "metadata": {
                        "name": "fake"
                    },
                    "spec": {
                        "replicas": 1,
                        "strategy": {
                            "rollingUpdate": {
                                "maxSurge": "50%",
                                "maxUnavailable": "50%"
                            },
                            "type": "RollingUpdate"
                        },
                        "template": {
                            "metadata": {
                                "labels": {
                                    "app": "fake"
                                }
                            },
                            "spec": {
                                "containers": [
                                    {
                                        "image": "mongo",
                                        "imagePullPolicy": "Always",
                                        "name": "container-0"
                                    }
                                ],
                                "restartPolicy": "Always",
                                "terminationGracePeriodSeconds": 40
                            }
                        }
                    }
                }
            },
            {
                title: 'with port',
                descriptor: {
                    metadata: {
                        name: 'fake',
                    },
                    spec: {
                        template: {
                            spec: {
                                containers: [
                                    {
                                        image: 'mongo',
                                        ports: [{
                                            containerPort: 9000,
                                            protocol: 'TCP'
                                        }
                                        ]

                                    }
                                ]
                            }
                        }
                    }
                },
                expectedObject: {
                    "apiVersion": "extensions/v1beta1",
                    "kind": "Deployment",
                    "metadata": {
                        "name": "fake"
                    },
                    "spec": {
                        "replicas": 1,
                        "strategy": {
                            "rollingUpdate": {
                                "maxSurge": "50%",
                                "maxUnavailable": "50%"
                            },
                            "type": "RollingUpdate"
                        },
                        "template": {
                            "metadata": {
                                "labels": {
                                    "app": "fake"
                                }
                            },
                            "spec": {
                                "containers": [
                                    {
                                        "image": "mongo",
                                        "imagePullPolicy": "Always",
                                        "name": "container-0",
                                        "ports": [
                                            {
                                                "containerPort": 9000,
                                                "protocol": "TCP"
                                            }
                                        ]
                                    }
                                ],
                                "restartPolicy": "Always",
                                "terminationGracePeriodSeconds": 40
                            }
                        }
                    }
                }
            },
            {
                title: 'with multiple ports',
                descriptor: {
                    metadata: {
                        name: 'fake',
                    },
                    spec: {
                        template: {
                            spec: {
                                containers: [
                                    {
                                        image: 'mongo',
                                        ports: [{
                                            containerPort: 9000,
                                            protocol: 'TCP'
                                        }, {
                                            containerPort: 9001,
                                            protocol: 'TCP'
                                        }
                                        ]

                                    }
                                ]
                            }
                        }
                    }
                },
                expectedObject: {
                    "apiVersion": "extensions/v1beta1",
                    "kind": "Deployment",
                    "metadata": {
                        "name": "fake"
                    },
                    "spec": {
                        "replicas": 1,
                        "strategy": {
                            "rollingUpdate": {
                                "maxSurge": "50%",
                                "maxUnavailable": "50%"
                            },
                            "type": "RollingUpdate"
                        },
                        "template": {
                            "metadata": {
                                "labels": {
                                    "app": "fake"
                                }
                            },
                            "spec": {
                                "containers": [
                                    {
                                        "image": "mongo",
                                        "imagePullPolicy": "Always",
                                        "name": "container-0",
                                        "ports": [
                                            {
                                                "containerPort": 9000,
                                                "protocol": "TCP"
                                            },
                                            {
                                                "containerPort": 9001,
                                                "protocol": "TCP"
                                            }
                                        ]
                                    }
                                ],
                                "restartPolicy": "Always",
                                "terminationGracePeriodSeconds": 40
                            }
                        }
                    }
                }
            },
            {
                title: 'with env',
                descriptor: {
                    metadata: {
                        name: 'fake',
                    },
                    spec: {
                        template: {
                            spec: {
                                containers: [
                                    {
                                        image: 'mongo',
                                        env: [{
                                            name: 'SIMPLE_ENV',
                                            value: 'VA1_1'
                                        }, {
                                            name: 'SECRET_ENV',
                                            valueFrom: {
                                                secretKeyRef: {
                                                    key: 'secret.name',
                                                    name: 'secret.name'
                                                }
                                            }
                                        }, {
                                            name: 'CONFIG_MAP_ENV',
                                            valueFrom: {
                                                configMapKeyRef: {
                                                    key: 'config.name',
                                                    name: 'config.name'
                                                }
                                            }
                                        }
                                        ]
                                    }
                                ]
                            }
                        }
                    }
                },
                expectedObject: {
                    "apiVersion": "extensions/v1beta1",
                    "kind": "Deployment",
                    "metadata": {
                        "name": "fake"
                    },
                    "spec": {
                        "replicas": 1,
                        "strategy": {
                            "rollingUpdate": {
                                "maxSurge": "50%",
                                "maxUnavailable": "50%"
                            },
                            "type": "RollingUpdate"
                        },
                        "template": {
                            "metadata": {
                                "labels": {
                                    "app": "fake"
                                }
                            },
                            "spec": {
                                "containers": [
                                    {
                                        "image": "mongo",
                                        "imagePullPolicy": "Always",
                                        "name": "container-0",
                                        "env": [
                                            {
                                                "name": "SIMPLE_ENV",
                                                "value": "VA1_1"
                                            },
                                            {
                                                "name": "SECRET_ENV",
                                                "valueFrom": {
                                                    "secretKeyRef": {
                                                        "key": "secret.name",
                                                        "name": "secret.name"
                                                    }
                                                }
                                            },
                                            {
                                                "name": "CONFIG_MAP_ENV",
                                                "valueFrom": {
                                                    "configMapKeyRef": {
                                                        "key": "config.name",
                                                        "name": "config.name"
                                                    }
                                                }
                                            }
                                        ]
                                    }
                                ],
                                "restartPolicy": "Always",
                                "terminationGracePeriodSeconds": 40
                            }
                        }
                    }
                }
            },
            {
                title: 'with resources',
                descriptor: {
                    metadata: {
                        name: 'fake',
                    },
                    spec: {
                        template: {
                            spec: {
                                containers: [
                                    {
                                        image: 'mongo',
                                        resources: {
                                            requests: {
                                                cpu: '200i',
                                                memory: '512Mb'
                                            },
                                            limits: {
                                                cpu: '200i',
                                                memory: '512Mb'
                                            }
                                        }
                                    }
                                ]
                            }
                        }
                    }
                },
                expectedObject: {
                    "apiVersion": "extensions/v1beta1",
                    "kind": "Deployment",
                    "metadata": {
                        "name": "fake"
                    },
                    "spec": {
                        "replicas": 1,
                        "strategy": {
                            "rollingUpdate": {
                                "maxSurge": "50%",
                                "maxUnavailable": "50%"
                            },
                            "type": "RollingUpdate"
                        },
                        "template": {
                            "metadata": {
                                "labels": {
                                    "app": "fake"
                                }
                            },
                            "spec": {
                                "containers": [
                                    {
                                        "image": "mongo",
                                        "imagePullPolicy": "Always",
                                        "name": "container-0",
                                        "resources": {
                                            "limits": {
                                                "cpu": "200i",
                                                "memory": "512Mb"
                                            },
                                            "requests": {
                                                "cpu": "200i",
                                                "memory": "512Mb"
                                            }
                                        }
                                    }
                                ],
                                "restartPolicy": "Always",
                                "terminationGracePeriodSeconds": 40
                            }
                        }
                    }
                }
            },
            {
                title: 'mark with labels',
                descriptor: {
                    metadata: {
                        name: 'fake',
                    },
                    spec: {
                        template: {
                            spec: {
                                containers: [
                                    {
                                        image: 'mongo',
                                        resources: {
                                            requests: {
                                                cpu: '200i',
                                                memory: '512Mb'
                                            },
                                            limits: {
                                                cpu: '200i',
                                                memory: '512Mb'
                                            }
                                        }
                                    }
                                ]
                            }
                        }
                    }
                },
                markWithLabels: true,
                keys: [{
                    path: 'spec.template.metadata.labels',
                    include: ['io.codefresh.auto-generated', 'io.codefresh.generated-at', 'app']
                }
                ]
            }
        ];
        tests.map((test) => {
            it(`with ${test.title}`, () => {
                const finalObject = new DeployBuilder(test.descriptor,
                    { markWithLabels: test.markWithLabels || false ,markAnnotations: test.markAnnotations || false}).build();
                if (!test.markWithLabels) {
                    expect(new Deploy(finalObject).toJson()).to.be.deep.equal(test.expectedObject);
                } else {
                    _.forEach(test.keys, (key) => {
                        expect(_.get(finalObject, key.path, {})).to.have.keys(key.include);
                    });
                }
            });
        });
    });

});