'use strict';

const _            = require('lodash');
const Promise      = require('bluebird');
const YAML         = require('js-yaml');
const EventEmitter = require('eventemitter2').EventEmitter2;
const CFError = require('cf-errors');

class RecourseController extends EventEmitter {

    constructor({
        kubeClient,
        kubeManager,
        selector,
        events,
        baseKubePath,
        Entity,
        builders,
        StatusAnalyzerClass,
        ns
    }) {
        super();
        this._kube                = kubeClient;
        this._selector            = selector;
        this._events              = events;
        this._baseKubePath        = baseKubePath;
        this._EntityClass         = Entity;
        this._builders            = builders || [];
        this._StatusAnalyzerClass = StatusAnalyzerClass;
        this._kubeManager         = kubeManager;
        this._ns                  = ns;
    }

    getKubeClient() {
        return this._kube;
    }

    getClient() {
        return _.get(this.getKubeClient(), this._baseKubePath);
    }

    getSelector() {
        return this._selector;
    }

    getBuilder(name) {
        let Builder;
        this._builders.map((builder) => {
            if (name && builder.name === name) {
                Builder = builder.Type
            }

            if (!name && builder.isDefault) {
                Builder = builder.Type
            }
        });
        if (Builder) {
            return Builder;
        }
        throw new CFError({
            message: `Builder ${name ? name : 'default'} not found`,
            recognized: true,
        });
    }

    get(filter) {
        const deferred = Promise.defer();
        this.getClient().get({ qs: filter }, (err, resourceList) => {
            const list = _.get(resourceList, 'items', []);
            if (err) {
                console.error(err)
                return deferred.reject(err);
            }
            Promise.map(list, (kubeResource) => {
                let status = {};
                if (this._StatusAnalyzerClass) {
                    status = new this._StatusAnalyzerClass(kubeResource,
                        resourceList.kind.replace('List', '')).analyze();
                }
                const entityInstance = new this._EntityClass(kubeResource, {
                    selector: this.getSelector(),
                    status
                });
                return entityInstance;
            })
                .then(_.compact)
                .then(deferred.resolve.bind(deferred))
                .catch(deferred.reject.bind(deferred))
                .done();
        });
        return deferred.promise;
    }

    describe(name) {
        const deferred = Promise.defer();
        _.get(this.getKubeClient(), this._baseKubePath)(name).get((err, kubeResource) => {
            if (err) {
                return deferred.reject(err);
            }
            let status = {};
            if (this._StatusAnalyzerClass) {
                status = new this._StatusAnalyzerClass(kubeResource, kubeResource.kind).analyze();
            }
            const entityInstance = new this._EntityClass(kubeResource, {
                selector: this.getSelector(),
                status
            });

            return deferred.resolve(entityInstance);

        });
        return deferred.promise;
    }

    create(resourceDescriptor) {
        const deferred = Promise.defer();

        const isYamlBased = !!_.get(resourceDescriptor, 'yaml', false) ||
                            _.isString(resourceDescriptor);
        let resourceTemplate;
        if (isYamlBased) {
            resourceTemplate = YAML.load(resourceDescriptor.yaml || resourceDescriptor);
        } else {
            const Builder    = this.getBuilder();
            resourceTemplate = new Builder(resourceDescriptor).build();
        }
        this.getClient().post({ body: resourceTemplate }, (err, resource) => {
            if (err) {
                return deferred.reject(err);
            }
            let status = {};
            if (this._StatusAnalyzerClass) {
                status = new this._StatusAnalyzerClass(resource, resource.kind).analyze();
            }
            let resourceInstance = new this._EntityClass(resource,
                { selector: this.getSelector(), status });

            this.publishEvent(this._events.types.ADDED, {
                name: resourceInstance.getName()
            });
            return deferred.resolve(resourceInstance)
        });


        return deferred.promise;
    }

    delete(name) {
        const deferred = Promise.defer();

        this.getClient().delete({ name: name }, (err, result) => {
            if (err) {
                return deferred.reject(err);
            }
            this._events && this.publishEvent(this._events.types.REMOVED, {
                name
            });
            return deferred.resolve(result);
        });
        return deferred.promise;
    }

    update(name, resourceDescriptor) {
        const deferred = Promise.defer();

        const isYamlBased = !!_.get(resourceDescriptor, 'yaml', false) ||
                            _.isString(resourceDescriptor);
        if (isYamlBased) {
            resourceDescriptor = YAML.load(resourceDescriptor.yaml || resourceDescriptor);
        }
        const resource = new this._EntityClass(resourceDescriptor,
            { selector: this.getSelector() });
        _.get(this.getKubeClient(), this._baseKubePath)(name)
            .put({ body: resource.toFullJson() }, (err, resource) => {
                if (err) {
                    return deferred.reject(err);
                }
                let status = {};
                if (this._StatusAnalyzerClass) {
                    status = new this._StatusAnalyzerClass(resource, resource.kind).analyze();
                }
                const entityInstance = new this._EntityClass(resource, {
                    selector: this.getSelector(),
                    status
                });
                this.publishEvent(this._events.types.UPDATED, {
                    name
                });
                return deferred.resolve(entityInstance);
            });
        return deferred.promise;
    }

    publishEvent(name, data) {
        this.emit(name, data);
    }

}

module.exports = RecourseController;
